module.exports.connected = connected;
module.exports.processMessage = processMessage;
module.exports.processSender = processSender;
module.exports.setHandlers = setHandlers;
module.exports.setSettings = setSettings;

var handlers = {
	sendHandler: null,
	findServer: null,
	findUser: null,
	killUser: null,
	newServer: null,
	newUser: null,
	parseUmodes: null,
	getChannel: null,
	quitUser: null
};

var settings = {
	ID: null,
	password: null,
	name: null,
	description: null,
	version: null,
	me: null
};

var prefixes = {
	'~': 'q',
	'&': 'a',
	'@': 'o',
	'%': 'h',
	'+': 'v'
};

function parseModePrefix(text){
	var modes = [];
	for(var i=0; i<text.length; i++){
		var c = text.charAt(i);
		if(c in prefixes){
			modes[prefixes[c]] = true;
		} else break;
	}
	var uid = text.substring(i);
	var user = handlers.findUser(uid);
	if(!user){
		handlers.killUser(uid);
		return null;
	}
	return { 'user': user, 'modes': modes };
}

function connected(){
	ircSend(null, settings.ID, 'PASS', [settings.password]);
	ircSend(null, settings.ID, 'PROTOCTL', ['NICKv2', 'VHP', 'UMODE2', 'NICKIP', 'SJOIN', 'SJOIN2', 'SJ3', 'NOQUIT', 'TKLEXT', 'MLOCK', 'SID']);
	ircSend(null, settings.ID, 'PROTOCTL', ['EAUTH=' + settings.name + ',,,' + settings.version]);
	ircSend(null, settings.ID, 'PROTOCTL', ['SID=' + settings.ID]);
	ircSend(null, settings.ID, 'SERVER', [settings.name, '1', settings.description]);
}

/*
		this.sender = {
			'nick': '',
			'ident': '',
			'host': '',
			'server': false,
			'user': false
		};
*/

function processSender(sender){
	if(sender.nick){
		if(server = handlers.findServer(sender.nick)){
			sender.nick = server.name;
			sender.server = server;
		} else if(user = handlers.findUser(sender.nick)){
			sender.nick = user.name
			sender.ident = user.ident;
			sender.host = user.host;
			sender.user = user;
		} else {
			handlers.killUser(sender.nick);
			return false;
		}
	}
	return sender;
}

function processMessage(msg){
	if(!msg) return;
	if(msg.command in cmdBinds){
		cmdBinds[msg.command](msg);
	} else {
		console.log('Unhandled cmd: '+msg.command);
		console.log(msg);
	}
}

function setHandlers(newHandlers){
	handlers = newHandlers;
}

function setSettings(newSettings){
	settings = newSettings;
}

function ircSend(tags, from, cmd, args){
	handlers.send(tags, from, cmd, args);
}

var cmdBinds = {
	'PROTOCTL': function(msg){
		console.log(msg);
	},
	
	'SERVER': function(msg){
		var expr = /^([^-]+)-([^-]+)-([^ ]+) (.*)$/;
		var match = expr.exec(msg.args[2]);
		if(match){
			handlers.newServer(msg.args[0], match[3], match[4], msg.args[1], settings.me);
		} else {
			throw 'Unknown SERVER message';
		}
	},
	
	'MD': function(msg){
	},
	
	'SMOD': function(msg){
	},
	
	'EOS': function(msg){
	},
	
	'SINFO': function(msg){
		console.log(msg);
	},

	'SID': function(msg){
		handlers.newServer(msg.args[0], msg.args[2], msg.args[3], msg.args[1], msg.sender);
	},
	
	'UID': function(msg){ // nick, distance, TS, ident, host, uid, account, umodes, vhost, cloakedHost, ip, realname, uplink
		handlers.newUser(msg.args[0], msg.args[1], msg.args[2], msg.args[3], msg.args[4], msg.args[5], msg.args[6], handlers.parseUmodes(msg.args[7]), msg.args[8], msg.args[9], msg.args[10], msg.args[11], msg.sender.server);
	},

	'SJOIN': function(msg){
		var channel = handlers.getChannel(msg.args[1]);
		channel.setTS(msg.args[0]);
		var modeArgs = [];
		for(var i=3; i<msg.args.length - 1; i++){
			modeArgs.push(msg.args[i]);
		}
		channel.addModes(msg.args[2], modeArgs);
		var members = msg.text.split(' ');
		for(var i=0; i<members.length; i++){
			var member = members[i];
			var c = member.charAt(0);
			if(c == '&'){ // +b
				channel.addBan(member.substring(1));
			} else if(c == '"'){ // +e
				channel.addExcept(member.substring(1));
			} else if(c == '\''){ // +I
				channel.addInvex(member.substring(1));
			} else {
				var udata = parseModePrefix(member);
				if(!udata) return;
				console.log(udata.user.name + ' joined ' + msg.args[1]);
				channel.joinUser(udata.user);
				channel.setStatusModes(udata.user, udata.modes);
			}
		}
		console.log(channel);
	},
	
	'SWHOIS': function(msg){
	},
	
	'TKL': function(msg){
	},
	
	'METADATA': function(msg){
	},
	
	'NETINFO': function(msg){
		console.log(msg);
	},
	
	'SASL': function(msg){
	},
	
	'PING': function(msg){
		msg.reply(null, 'PONG', msg.args);
	},

	'QUIT': function(msg){
		handlers.quitUser(msg.sender.user);
	},

	'PART': function(msg){
		var channel = handlers.getChannel(msg.args[0]);
		channel.removeUser(msg.sender.user);
	}
};

